package com.yihu.hos.arbiter.services;

import com.yihu.hos.arbiter.configuration.ArbiterServerConfiguration;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.web.framework.constant.EndPointConstant;
import com.yihu.hos.web.framework.constant.MycatConstant;
import com.yihu.hos.web.framework.constant.SSHConstant;
import com.yihu.hos.web.framework.constant.ServiceFlowConstant;
import io.zbus.mq.*;
import org.apache.camel.CamelContext;
import org.apache.camel.ProducerTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Airhead
 * @since 2016/12/22.
 */
@Service
public class ProxyService {
    private static final Logger logger = LoggerFactory.getLogger(ProxyService.class);

    private CamelContext camelContext;
    private ArbiterServerConfiguration configuration;
    private Broker broker;

    @Autowired
    public void setBroker(Broker broker) {
        this.broker = broker;
    }

    @Autowired
    public void setCamelContext(CamelContext camelContext) {
        this.camelContext = camelContext;
    }

    @Autowired
    public void setConfiguration(ArbiterServerConfiguration configuration) {
        this.configuration = configuration;
    }

    public void start() {
        new ServiceFlow().start();
        new Mycat().start();
        new Shell().start();
        new EndPoint().start();
    }

    public class ServiceFlow {
        private Consumer consumer;

        public void start() {
            if (broker == null) {
                logger.error("broker is null");
                return;
            }

            try {
                ConsumerConfig config = new ConsumerConfig(broker);
                config.setTopic(ServiceFlowConstant.ZBUS_MQ + "@" + configuration.getTenant());
                ConsumeGroup consumeGroup = ConsumeGroup.createTempBroadcastGroup();
                config.setConsumeGroup(consumeGroup);
                config.setMessageHandler((message, mqClient) -> {
                    Map<String, Object> header = new HashMap<>();
                    header.put("event", message.getHeader("event"));

                    ProducerTemplate producerTemplate = camelContext.createProducerTemplate();
                    producerTemplate.sendBodyAndHeaders(ServiceFlowConstant.CAMEL_ENDPOINT, message.getBodyString(), header);
                });

                consumer = new Consumer(config);
                consumer.start();

            } catch (IOException e) {
                e.printStackTrace();
                logger.error(e.getMessage());
            }
        }

        @Override
        protected void finalize() throws Throwable {
            if (consumer != null) {
                consumer.close();
            }

            super.finalize();
        }
    }

    public class Mycat {
        private Consumer consumer;

        public void start() {
            if (broker == null) {
                logger.error("broker is null");
                return;
            }

            try {
                ConsumerConfig config = new ConsumerConfig(broker);
                config.setTopic(MycatConstant.ZBUS_MQ + "@" + configuration.getMycatName());
                ConsumeGroup consumeGroup = ConsumeGroup.createTempBroadcastGroup();
                config.setConsumeGroup(consumeGroup);
                config.setMessageHandler((message, mqClient) -> {
                    Map<String, Object> header = new HashMap<>();
                    header.put("event", message.getHeader("event"));

                    ProducerTemplate producerTemplate = camelContext.createProducerTemplate();
                    producerTemplate.sendBodyAndHeaders(MycatConstant.CAMEL_ENDPOINT, message.getBodyString(), header);
                });

                consumer = new Consumer(config);
                consumer.start();
            } catch (IOException e) {
                e.printStackTrace();
                logger.error(e.getMessage());
            }
        }

        @Override
        protected void finalize() throws Throwable {
            if (consumer != null) {
                consumer.close();
            }

            super.finalize();
        }
    }


    public class Shell {
        private Consumer consumer;

        public void start() {
            if (broker == null) {
                logger.error("broker is null");
                return;
            }
            try {
                ConsumerConfig config = new ConsumerConfig(broker);
                config.setTopic(SSHConstant.SHELL_REQUEST + "@" + configuration.getTenant());
                ConsumeGroup consumeGroup = ConsumeGroup.createTempBroadcastGroup();
                config.setConsumeGroup(consumeGroup);
                config.setMessageHandler((message, mqClient) -> {
                    Map<String, Object> header = new HashMap<>();
                    header.put("event", message.getHeader("event"));

                    ProducerTemplate producerTemplate = camelContext.createProducerTemplate();
                    producerTemplate.sendBodyAndHeaders(SSHConstant.CAMEL_ENDPOINT, message.getBodyString(), header);
                });

                consumer = new Consumer(config);
                consumer.start();
            } catch (IOException e) {
                e.printStackTrace();
                logger.error(e.getMessage());
            }
        }

        @Override
        protected void finalize() throws Throwable {
            if (consumer != null) {
                consumer.close();
            }

            super.finalize();
        }
    }

    public class EndPoint {
        private Consumer consumer;

        public void start() {
            if (broker == null) {
                logger.error("broker is null");
                return;
            }
            try {
                ConsumerConfig config = new ConsumerConfig(broker);
                config.setTopic(EndPointConstant.ZBUS_MQ + "@" + configuration.getTenant());
                ConsumeGroup consumeGroup = ConsumeGroup.createTempBroadcastGroup();
                config.setConsumeGroup(consumeGroup);
                config.setMessageHandler((message, mqClient) -> {
                    Map<String, Object> header = new HashMap<>();
                    header.put("event", message.getHeader("event"));

                    ProducerTemplate producerTemplate = camelContext.createProducerTemplate();
                    producerTemplate.sendBodyAndHeaders(EndPointConstant.CAMEL_ENDPOINT, message.getBodyString(), header);
                });

                consumer = new Consumer(config);
                consumer.start();
            } catch (IOException e) {
                e.printStackTrace();
                logger.error(e.getMessage());
            }
        }

        @Override
        protected void finalize() throws Throwable {
            if (consumer != null) {
                consumer.close();
            }

            super.finalize();
        }
    }
}
