package com.yihu.hos.arbiter.services;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.hos.core.http.HTTPResponse;
import com.yihu.hos.core.http.HttpClientKit;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.web.framework.constant.SSHConstant;
import com.yihu.hos.web.framework.model.bo.BrokerServer;
import com.yihu.hos.web.framework.model.bo.ServiceShell;
import io.zbus.mq.Broker;
import io.zbus.mq.Message;
import io.zbus.mq.Producer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author HZY
 * @vsrsion 1.0
 * Created at 2017/1/5.
 */
@Service("linuxShellService")
public class ShellService {
    private static final Logger logger = LoggerFactory.getLogger(ShellService.class);

    @Autowired
    private BrokerServerService brokerServerService;
    private Broker broker;
    @Autowired
    private ObjectMapper objectMapper;

    /**
     * 发送shell命令请求到broker
     *
     * @param msg 命令消息内容
     */
    public void sendShell(String msg) {
        try {
            List<BrokerServer> brokerServerList = brokerServerService.get(false);
            for (BrokerServer broker : brokerServerList) {
                String result = sendMessage(broker, "post", "/esb/serviceShell/send", msg);
                if (result == null) {
                    logger.error("sendMessage to broker start failed, broker:" + broker.getURL() + ", msg:" + msg);
                    continue;
                } else {
                    logger.debug("发送shell请求到broker成功");
                    // shell执行成功，见执行结果返回到中心zbus显示。
                    ServiceShell serviceShell = objectMapper.readValue(msg, ServiceShell.class);
                    String topic = SSHConstant.SHELL_RESPONSE + "@" + serviceShell.getTenant();
                    Producer producer = new Producer(this.broker);
                    producer.declareTopic(topic);
                    Message message = new Message();
                    message.setTopic(topic);
                    message.setHeader("event", serviceShell.getType());
                    message.setHeader("tenant", serviceShell.getTenant());
                    message.setMethod("POST");
                    message.setBody(result);
                    Message res = producer.publish(message);

                    if (logger.isInfoEnabled()) {
                        logger.info(res.toString());
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error(e.getMessage());
        }
    }

    @Autowired
    public void setBroker(Broker broker) {
        this.broker = broker;
    }

    private String sendMessage(BrokerServer brokerServer, String method, String path, String msg) {
        if (brokerServer == null) {
            return null;
        }

        switch (method) {
            case "post": {
                HTTPResponse response = HttpClientKit.post(brokerServer.getURL() + path, msg);
                if (response.getStatusCode() == 200) {
                    String body = response.getBody();
                    return body;
                }

                return null;
            }

            case "put": {
                HTTPResponse response = HttpClientKit.put(brokerServer.getURL() + path, msg);
                if (response.getStatusCode() == 200) {
                    String body = response.getBody();
                    logger.debug(body);
                    return body;
                }

                return null;
            }


            case "delete": {
                HTTPResponse response = HttpClientKit.delete(brokerServer.getURL() + path, msg);
                if (response.getStatusCode() == 200) {
                    String body = response.getBody();
                    logger.debug(body);
                    return body;
                }

                return null;
            }

            default:
                break;
        }

        return null;
    }

}
